# Distribution Checker
# Function Compliance Checker Integration Module (Func_Compliance_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Func_Compliance_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common
#----------------------------------------------------------------------

my $checkpoints = undef;

#----------------------------------------------------------------------

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/lsb-functions-signature-checker";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	my $run_script = $self->option('RUN_SCRIPT') || "./lsb-functions-signature-checker.pl";
	$self->{RUN_SCRIPT} = $run_script;
	my $results_dir = $self->option('RESULTS_DIR') || $testsuite_dir."/results";
	$self->{RESULTS_DIR} = $results_dir;
	$self->{TEST_DATA} = $self->option('TEST_DATA') || "/opt/moblin/test/moblin-functions-signature-checker-data";

	# Remove old results
	cmd("rm -rf ".shq($self->{RESULTS_DIR})."/*");

	init_checkpoint_data($self->{VERSION});

	return 1;
}

sub run {
	my ($self) = @_;
	
	# Open a subshell.
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Go to the test temp directory
	$subshell->Settle();
	$subshell->Send("cd ".shq($self->{TESTSUITE_DIR})."\n");

	my $distr_name = $host_info->{'OS'};
	$distr_name =~ s/[^\w\_\-.,^=]/-/g; # TODO: remove when fixed in lsb-functions-signature-checker
	$distr_name =~ s/^-+//; # remove '-' at the beginning for not be mistaken with an option.

	# Prepare the command
	my $command = shq($self->{RUN_SCRIPT})
		." -run"
		." -src ".shq($self->{TEST_DATA})
		." -distr ".shq($distr_name)
		;

	$subshell->Settle();
	$subshell->Send("$command; exit\n"); # Run the test
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		$self->check_progress($line);
	}
	
	# Just confirm that the subshell has finished
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /^testing for (\S+) ABI compliance.*[\r\n]/ ) {
		my $testcase = $1;
		$self->progress_profiled($checkpoints, $testcase);
		return;
	}
	# else:
	$self->progress_profiled();
}

sub after_run {
	my ($self) = @_;

	$self->{JOURNAL_FILE} = $self->name.".simplejournal";
	$self->{JOURNAL_TYPE} = 'SIMPLE';

	# Find the journal file and copy it to where it should be.
	is_ok $self->copy_file( $self->{RESULTS_DIR}."/journal", $self->journal_file )
		or $self->report_error(complain $Error::Last);
	
	is_ok $self->copy_file( $self->{RESULTS_DIR}."/abi_compliance_report.html", $globals->{"test_result_dir"} )
		or $self->report_error(complain $Error::Last);
	is_ok $self->copy_file( $self->{RESULTS_DIR}."/compat_reports", $globals->{"test_result_dir"} )
		or $self->report_error(complain $Error::Last);

	$self->report( '<p>Open '.Report::local_href( "results/abi_compliance_report.html", 'full report' ).'.</p>' );

	return 1;
}

sub init_checkpoint_data {
	my ($ver) = @_;
	
	# moblin-functions-signature-checker-data 4.0.0-1 for Moblin 2.0
	$checkpoints = [
		
	];
}

1; # return value
