# Distribution Checker
# Qt4-azov Test Module (Qt4_azov_test.pm)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
# The ptyshell tool is originally written by Jiri Dluhos <jdluhos@suse.cz>
# Copyright (C) 2005-2007 SuSE Linux Products GmbH
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Qt4_azov_test;
use strict;

use Misc;
use Subshell;

use Test_common;
our @ISA = qw(Test_common); # Inherit Test_common

use X11_common;
#----------------------------------------------------------------------

my $checkpoints = undef;

#----------------------------------------------------------------------

sub check {
	my ($self) = @_;
	
	if ( does_shell_know ("ldconfig") ) {
		my @missing_libs = ();
		# !!! Keep this list in sync with the suggest_packages() sub !!!
		for my $lib ( qw/libQtCore.so.4 libQtGui.so.4 libQtSql.so.4 libQtSvg.so.4
					 libQtOpenGL.so.4 libQtXml.so.4 libQtNetwork.so.4 / )
		{
			my $lib_grep = "\\<".$lib." ";
			$lib_grep =~ s/\./\\./g;
			if ( cmd("ldconfig -p | grep ".shq($lib_grep)) != 0 ) {
				push @missing_libs, $lib;
			}
		}
		if ( @missing_libs ) {
			return error missing_packages_message("Can't find some required libraries, please install them:\n ", @missing_libs);
		}
	}
	
	# Do not check X11 fonts. They are provided by 'lsb-xvfb' package that will be installed later.
	
	return 1; # OK
}

sub prepare {
	my ($self) = @_;

	# Call the parent's prepare function
	is_ok $self->Test_common::prepare() or return $Error::Last;
	
	cmd("killall Xvfb"); # Should be done before calling guess_display()
	sleep 1;
	
	( defined $self->{OPTIONS} )
		or return error "Answers are undefined"; # Should not happen
	
	my $testsuite_dir = $self->option('TESTSUITE_DIR') || "/opt/lsb/test/qt4-azov";
	$self->{TESTSUITE_DIR} = $testsuite_dir;
	$self->{RESULTS_DIR} = $self->option('RESULTS_DIR') || "/opt/lsb/test/qt4-azov/results";
	
	# NOTE: X11 fonts are provided by 'lsb-xvfb' package.
	my $q = 'X11_FONT_PATH';
	if ( !defined $self->{OPTIONS}{$q}->{VALUE} ) {
		my $x11_fonts_dir = X11_common::guess_font_dir();
		is_ok($x11_fonts_dir) or return error $self->name
			." test requires certain X11 fonts to be installed.", $Error::Last;
		$self->{OPTIONS}{$q}->{VALUE} = $x11_fonts_dir."/misc";
	}
	
	# Remove old results
	cmd("rm -rf ".shq($self->{RESULTS_DIR})."/*");
	
	init_checkpoint_data($self->{VERSION});
	
	return 1;
}

sub run {
	my ($self) = @_;
	
	my $subshell = $self->Spawn_subshell();
	is_ok $subshell or return $Error::Last;
	
	# Change to the test suite directory
	$subshell->Settle();
	$subshell->Send("cd ".shq($self->{TESTSUITE_DIR})."\n");
	
	my $test_display = ":9.0";
	my $x11_fonts_dir = $self->option('X11_FONT_PATH')
		or return error "X11 font dir is not set";
	
	# Start X Virtual Frame Buffer
	$subshell->Settle();
	$subshell->Send( "/opt/lsb/bin/Xvfb"
			." -render -screen 0 1024x768x24"
			." ".$test_display
			." -ac +bs -kb -fp"
			." ".shq($x11_fonts_dir)
			." &"."\n" );
	
	$subshell->Settle();
	$subshell->Send("export DISPLAY=$test_display"."\n");
	
	$subshell->Settle();
	$subshell->Send("export G_SLICE=always-malloc"."\n");
	
	# Run the test
	$subshell->Settle();
	$subshell->Send("./run_tests.sh; exit"."\n");
	
	# Just wait for the test finished
	while ( my $line = $subshell->Read(2) ) {
		$self->check_progress($line);
	}
	
	# Just confirm that the subshell has finished
	$subshell->WaitForSubshell();
	
	if ( $subshell->{EXIT_CODE} ) {
		$self->report_warning(warning "subshell exited with code ".$subshell->{EXIT_CODE}
			.".\n".$subshell->get_last_lines);
	}
	
	return 1; # Ok
}

sub check_progress {
	my ($self, $line) = @_;
	
	defined $line or $line = "";
	
	if ( $line =~ /^Executing tests for (.*)[\r\n]/ ) {
		my $testcase = $1;
		$self->progress_profiled($checkpoints, $testcase);
		return;
	}
	# else:
	$self->progress_profiled();
}

sub after_run {
	my ($self) = @_;
	
	$self->{JOURNAL_FILE} = $self->name.".journal";
	$self->{JOURNAL_TYPE} = 'TET';
	
	# Find the journal file and copy it to where it should be.
	is_ok $self->copy_file_globbed( $self->{RESULTS_DIR}."/0*/journal", $self->journal_file )
		or $self->report_error(complain $Error::Last);
	
	return 1;
}

sub Cleanup {
	my ($self) = @_;
	
	cmd ("killall -q Xvfb");
}

sub init_checkpoint_data {
	my ($ver) = @_;
	
	# qt4-azov v. 4.0.0-1 for LSB 4.0
	if ( $ver le "4.0.0-1" ) {
		$checkpoints = [
			[ 0, 'QSessionManager' ],
			[ 6, 'QGraphicsSceneMouseEvent' ],
			[ 12, 'QAccessibleApplication' ],
			[ 18, 'QColorDialog' ],
			[ 25, 'QProcess' ],
			[ 32, 'QtPrivate' ],
			[ 38, 'QMenu' ],
			[ 44, 'QItemSelectionRange' ],
			[ 50, 'QTableWidget' ],
			[ 60, 'Functions' ],
			[ 66, 'QGLFramebufferObject' ],
			[ 72, 'QSqlField' ],
			[ 79, 'QTextFormat' ],
			[ 85, 'QToolTip' ],
			[ 91, 'QListWidgetItem' ],
			[ 97, 'QXmlAttributes' ],
		];
	} else {
		# qt4-azov v. 4.0.0-7
		$checkpoints = [
			[ 0, 'QSessionManager' ],
			[ 6, 'QGraphicsSceneDragDropEvent' ],
			[ 12, 'QAccessibleInterface' ],
			[ 19, 'QProgressDialog' ],
			[ 25, 'QFontMetricsF' ],
			[ 31, 'QAbstractFileEngine' ],
			[ 37, 'QApplication' ],
			[ 43, 'QAbstractTextDocumentLayout' ],
			[ 49, 'QTableWidgetSelectionRange' ],
			[ 55, 'QSound' ],
			[ 61, 'QNetworkInterface' ],
			[ 68, 'QSplitterHandle' ],
			[ 74, 'QSqlError' ],
			[ 80, 'QTextCursor' ],
			[ 86, 'QProgressBar' ],
			[ 92, 'QRadioButton' ],
			[ 98, 'QDomNode' ],
		];
	}
}

1; # return value
